const Generator = require('yeoman-generator');

module.exports = class extends Generator {
    constructor(args, opts) {
        super(args, opts)
    }
    async prompting() {
        this.ctx = await this.prompt([
            {
                type: "input",
                name: "projectName",
                message: "请输入工程名",
                default: 'koa-project'
            },
            {
                type: "input",
                name: "desc",
                message: "请输入项目描述"
            },
            {
                type: 'input',
                name: 'author',
                message: '请输入开发者姓名'
            },
            {
                type: "list",
                name: "template",
                message: "请选择一种模版",
                choices: [
                    { name: 'ejs', value: 'ejs' },
                    { name: 'jade', value: 'jade' }
                ]
            },
            {
                type: 'input',
                name: 'port',
                message: '请指定项目端口号',
                default: '3000'
            },
            {
                type: 'input',
                name: 'staticPath',
                message: '请指定静态资源目录',
                default: 'assets'
            }
        ]);
    }
    writing() {
        // 这里是generator特殊文件（路由，控制器，等）
        // 文件工具
        const { ctx } = this;
        const copy = (path, opts) => {
            if (opts) {
                this.fs.copyTpl(this.templatePath(path), this.destinationPath(path), opts)
            } else {
                this.fs.copy(this.templatePath(path), this.destinationPath(path), opts)
            }
        }
        copy('package.json', ctx);
        copy('package-lock.json');
        copy('README.md');
        copy('app.js', ctx);
        copy('utils/log4js.js', ctx);
        copy('utils/index.js');
        copy('utils/request.js');
        copy('config/index.js');
        copy('config/conf.development.js', ctx);
        copy('config/conf.production.js', ctx);
        copy('config/conf.test.js', ctx);
        copy('controller/index.js');
        copy('controller/example.js');
        copy('middlewares/index.js');
        copy('middlewares/handler.js');
        copy('routes/index.js');
        if (ctx.template === 'ejs') {
            copy('views/example.ejs')
            copy('views/error/404.ejs');
            copy('views/error/500.ejs');
        }else if( ctx.template === 'jade'){
            copy('views/example.jade')
            copy('views/error/404.jade');
            copy('views/error/500.jade');
        }
        copy('services/index.js');
        copy('services/example.js');
    }
    install() {
        this.installDependencies();
    }
    end() {
        console.log(`现在可以运行 “npm run dev” 并访问 http://127.0.0.1:${this.ctx.port}/example`)
    }
};