const Koa = require("koa");
const path = require("path");
const routes = require("./routes");
const views = require('koa-views');
const koaStatic = require("koa-static");
const bodyParser = require('koa-bodyparser');
const log4js = require("./utils/log4js");
const Middle = require("./middlewares");
const CONF = require("./config");
const app = new Koa();

//判断是否开发环境
const isDev = !/development/.test(process.env.NODE_ENV);

//配置模板引擎
app.use(views(path.join(__dirname, './views'), {
    extension: "<%= template %>"
}))

app.use(koaStatic(path.join(__dirname, "<%= staticPath %>")));  //静态资源中间件

app.use(Middle.Handler.base);       //请求基础数据添加
app.use(Middle.Handler._500);       //status 500 拦截
app.use(Middle.Handler._404);       //status 404 拦截
app.use(Middle.Handler.logs);       //日志打印

//参数解析中间件
app.use(bodyParser());

//应用路由
app.use(routes.routes());

//应用程序错误监听
app.on("error", (err, ctx) => {
    log4js.error(err);
});

app.listen(CONF.PORT, () => {
    log4js.info("运行环境", process.env.NODE_ENV);
    log4js.info("listen port", CONF.PORT)
});