const log4js = require("../utils/log4js");

/**
 * == 404 异常捕获 ==
 * @param {*} ctx 
 * @param {*} next 
 */
exports._404 = async (ctx, next) => {
    await next();
    if (ctx.status == "404") {
        log4js.warn(`访问地址不存在：${ctx.url}`);
        await ctx.render("error/404");
    }
}

/**
 * == 服务异常捕获 ==
 * @param {*} ctx 
 * @param {*} next 
 */
exports._500 = async (ctx, next) => {
    try {
        await next();
    } catch (err) {
        log4js.error("服务异常：", err);
        await ctx.render("error/500");
    }
}

/**
 * == 请求日志打印 ==
 * @param {*} ctx 
 * @param {*} next 
 */
exports.logs = async (ctx, next) => {
    const start = Date.now();
    await next();
    const ms = Date.now() - start;
    log4js.info(`${ctx.method} ${ctx.url}- ${ms / 1000}s`);
}


/**
 * == 基础数据添加 ==
 * @param {*} ctx 
 * @param {*} next 
 */
exports.base = async (ctx, next) => {
    const { query = {} } = ctx;
    Object.assign(ctx.state, {
        params: query,
        systemTime: +new Date(),
    });
    await next();
}


/**
 * == pc端渲染 ==
 * @param {*} ctx 
 * @param {*} next 
 */
exports.render = async (ctx, next) => {
    const { tplPath = "" } = ctx.state;
    if (tplPath)
        await ctx.render(tplPath);
    else
        await next();
}

